from vessl.internal.vessl_run import VesslRun

_run = VesslRun()

init = _run.init
api = _run.api
finish = _run.finish
log = _run.log
upload = _run.upload
progress = _run.progress
hyperparameters = _run.hyperparameters
